
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>

#define NEWEXE_SIGNATURE 0x454E

#pragma pack(1)
typedef unsigned short UINT16;
typedef unsigned long UINT32;
typedef unsigned char UINT8;

typedef struct
{
 unsigned     dgroupType:2;
 unsigned     globalInit:1;
 unsigned     protectedMode:1;
 unsigned     i86Instructions:1;
 unsigned     i286Instructions:1;
 unsigned     i386Instructions:1;
 unsigned     i87Instructions:1;
 unsigned     applicationType:3;
 unsigned     familyApplication:1;
 unsigned     unused1:1;
 unsigned     linkErrors:1;
 unsigned     unused2:1;
 unsigned     libraryModule:1;
} MODULEBITFLAGS;

typedef union
{
  MODULEBITFLAGS b;
  UINT16   i;
} FLAGWORD;

typedef struct
{
  UINT16    signature;
  UINT8     link_version;       // Version of link used
  UINT8     link_revision;      // Revision level of link used
  UINT16    entry_offset;       // Offset to entry table
  UINT16    entry_size;         // Size of entry table
  UINT32    crc_32;             // CRC-32 for file load control
  FLAGWORD  flag_word;          // Control for DOS loading
  UINT16    auto_data_seg;      // Index to auto data segment
  UINT16    org_heap_size;      // Original heap allocation size
  UINT16    org_stack_size;     // Original stack size
  UINT32    entry_point;        // CS:IP of entry point
  UINT32    start_stack;        // SS:SP of beginning stack
  UINT16    segment_count;      // # of entries in Segment Table
  UINT16    module_count;       // # of entries in Module Table
  UINT16    non_res_table_size; // Non-Resident name table size
  UINT16    segment_offset;     // Offset to Segment Table
  UINT16    resource_offset;    // Offset to Resource Table
  UINT16    resident_offset;    // Offset to Resident Name Table
  UINT16    module_offset;      // Offset to Module Table
  UINT16    import_offset;      // Offset to imported name table
  UINT32    nonres_name_offset; // Offset to non-res name table
  UINT16    entry_count;        // Number of movable entry point
  UINT16    shift_count;        // logical sector alignment
  UINT16    resource_count;     // number of resources
  UINT8     ne_exetyp;          // Target operating system
  UINT8     ne_flagsother;      // Other .EXE flags
  union
  {
    UINT16 ne_pretthunks;       // offset to return thunks
    UINT16 ne_gang_start;       // start of gangload area
  } x;
  union
  {
    UINT16 ne_psegrefbytes;     // offset to segment ref. bytes
    UINT16 ne_gang_length;      // length of gangload area
  } y;
  UINT16 ne_swaparea;           // minimum code swap area size
  UINT16 ne_expver;             // expected windows version num
} NEWEXEHEADER;



void main(int argc,char *argv[])
{

   FILE *file;
   NEWEXEHEADER nehdr;
   long nehdr_offset, curr_offset;
   char *exe=NULL;
   int i ;
   UINT16 ordinal,zero=0;
   UINT8 length;
   char buffer[256];

   if(argc <= 1)
     goto help ;
   else
    for(i=1 ; i < argc ; i++)
    {
      if(*argv[i] == '/' || *argv[i] == '-')
        {
            switch(argv[i][1])
            {
               case '?':
               case 'h':
               case 'H':
               default:
                    /* Help text */
                  help:
                    fprintf(stderr,"Usage: %s filename\n",argv[0]) ;
                    exit(0) ;
            }
        }
      else
         exe=argv[i] ;

    }

    if(exe == NULL || (file = fopen(exe, "r+b")) == NULL)
        fprintf(stderr,"\007Error opening file:%s\n", (exe==NULL) ? "" : exe) ;
      else
      {
       fseek(file, 0x3C, SEEK_SET);
       if(fread(&nehdr_offset, sizeof(nehdr_offset), 1, file) != 1 ||
          nehdr_offset <= 0x3C)
        {
           fclose(file) ;
           fprintf(stderr,"Error reading offset to header\n") ;
           exit(1) ;
        }
       fseek(file, nehdr_offset, SEEK_SET);
       if(fread(&nehdr, sizeof(nehdr), 1, file) != 1 ||
          nehdr.signature != NEWEXE_SIGNATURE )
         {
            fclose(file) ;
            fprintf(stderr,"Error reading %s's NE header\n",exe) ;
            exit(1) ;
         }


       if(nehdr.resident_offset)
       {

         fseek(file, nehdr.resident_offset + nehdr_offset, SEEK_SET);

         if(fread(&length, sizeof(length), 1, file) == 1 && length != 0)
          {
             fread(buffer, (UINT16)length, 1, file);
             buffer[length] = 0;
             curr_offset=ftell(file) ;
             fread(&ordinal, sizeof(ordinal), 1, file);
             if(ordinal != 0)
             {
               fseek(file, curr_offset, SEEK_SET);
               if(fwrite(&zero, sizeof(zero), 1, file) == 1)
                 printf("%s: ordinal %u for module \"%s\" has been reset to zero\n",
                         exe,ordinal,buffer) ;
                else
                  fprintf(stderr,"Error rewriting module name's ordinal for %s\n",exe) ;
             }
            else
             printf("%s: module name \"%s\" is already ordinal zero\n",
                       exe,buffer) ;
           }
          else
           fprintf(stderr,"Error reading resident name table for %s\n",exe) ;

         if(nehdr.non_res_table_size && nehdr.nonres_name_offset)
          {
            fseek(file, nehdr.nonres_name_offset, SEEK_SET);

            if(fread(&length, sizeof(length), 1, file) == 1 &&
               length != 0 && (UINT16)length <= nehdr.non_res_table_size-3)
              {
                 fread(buffer, (UINT16)length, 1, file);
                 buffer[length] = 0;
                 curr_offset=ftell(file) ;
                 fread(&ordinal, sizeof(ordinal), 1, file);
                 if(ordinal != 0)
                 {
                   fseek(file, curr_offset, SEEK_SET);
                   if(fwrite(&zero, sizeof(zero), 1, file) == 1)
                     printf("%s: ordinal %u for description \"%s\" has been reset to zero\n",
                             exe,ordinal,buffer) ;
                    else
                      fprintf(stderr,"Error rewriting description's ordinal for %s\n",exe) ;
                 }
                else
                 printf("%s: description \"%s\" is already ordinal zero\n",
                           exe,buffer) ;
               }
              else
               fprintf(stderr,"Error reading non-resident name table for %s\n",exe) ;

          }
         else
          printf("%s: No description contained in this file\n",exe) ;
       }
      else
        fprintf(stderr,"Error locating resident name table for %s\n",exe) ;

      fclose(file) ;
     }

    exit(0) ;
}
